var gallerydb = require("../model/gallerydb"),
    im = require('imagemagick'), 
    utils = require('../lib/utils'),
    _ = require('underscore');




function GalleryManager(dbname, imageCache) {
  this.galleryDb = new gallerydb.GalleryDB({dbname: dbname});

  _.bindAll(this);
};

GalleryManager.prototype.createUser = function(username, email) {
  var user =  {
    username: username,
    email: email,
    displayName: '',
    description: '',
    location: '',
    thumbnail: ''
  };
  
  return this.galleryDb.insertUser(user);
};

GalleryManager.prototype.findUserByUsername = function(username, callback) {
  this.galleryDb.findUserByUsername(username, callback);

  return this;  
};

// 
// Photo methods

GalleryManager.prototype.upload = function(user, imagePath, callback) {
  var self = this;
  var callbacks = utils.Callbacks.toObjectCallbacks(callback);

  // todo: move photo to image file store

  GalleryManager_getMetaData(this, imagePath, {
      error: callbacks.error,
      success: function (metadata) {

        var photo = {
          user: user,
          title: "TODO",
          description: "TODO",
          taken: metadata.taken,

          images: {
             original: new PhotoSrc(imagePath, metadata.width, metadata.height)
           }

        };

        callbacks.success(self.galleryDb.insertPhoto(photo));
        
      }
  });
  
  return this;
};


GalleryManager.prototype.findPhotoImageById = function(id, callback) {
   this.galleryDb.findPhotoImageById(id, callback);
}


/**
 * NOTE:  This is fully distructive.  All persisted data will be deleted!
 */
GalleryManager.prototype.dropData = function(callback) {
   this.galleryDb.dropAll(callback);
   
   return this;
};

GalleryManager.prototype.shutdown = function(callback) {
  fn = callback || function() {};
  
  this.galleryDb.disconnect(fn);
  
  return this;
};

// GalleryManager privates

  function PhotoSrc(src, width, height) {
    this.src = src;
    this.width = width;
    this.height = height;
  }


var scaleWidths = {
  'thumb':   100,
  'small':   320,
  'medium':  640,
  'large':   1024,
}

function GalleryManager_resizePhoto(self, imagePath, imageBaseName, size, callbacks) {
  var resizedPhotoPath =  imageBaseName + '-' + size + '.jpg'; 

  im.resize(
    {
      srcPath: imagePath,
      dstPath: resizedPhotoPath,
      width:   scaleWidths[size] 
    }, 
    utils.Callbacks.toStandardCallback({
      error: callbacks.error,
      success: function() {
        im.identify(resizedPhotoPath,   
          utils.Callbacks.toStandardCallback({
            error: callbacks.error,
            success: function(features) {
               callbacks.success({
                  imagePath: resizedPhotoPath,
                  width: features.width,
                  heigth: features.height
              });
            }
          }));         
      }
    }));
}

function GalleryManager_getMetaData(self, imagePath, callbacks) {
  im.readMetadata(imagePath, utils.Callbacks.toStandardCallback({
    error: callbacks.error,
    success: function(metadata) {
      if (!metadata.exif) {
        // default to simple data
        im.identify(imagePath, utils.Callbacks.toStandardCallback({
          error: callbacks.error,
          success: function(basicInfo) {
             callbacks.success({
                width: basicInfo.width,
                height: basicInfo.height,
                taken: new Date()
             });
          }
        }));
      } else {
        callbacks.success({
            width: metadata.exif.exifImageWidth,
            height: metadata.exif.exifImageLength,
            taken: metadata.exif.dateTimeOriginal
        });
      }

    }
  }));
}

//
// Exports

module.exports = {
  GalleryManager: GalleryManager
};