var mongoose  = require("mongoose"), 
    _ = require('underscore');
    
//    
// Schemas    
var Schema = mongoose.Schema;    

var UserSchema = new Schema({
  username: String,
  email: String,
  displayName: String,
  description: String,
  thumbnail: String,
  location: String
});

var PhotoImageSchema = new Schema({
  src: String,
  width: {type: Number, min: 0},
  height: {type: Number, min: 0}
});

var PhotoSchema = new Schema({  
    user: Schema.ObjectId, // User
    title: { type: String, required: false },  
    description: { type: String, required: false },  
    privacy: { type: String, required: true, 'default': "PRIVATE" },
    taken: {type: Date, required: true},
    uploaded: { type: Date, 'default': Date.now, required: true },
    images: {
      // PhotoImages
      thumbnail: Schema.ObjectId,
      small: Schema.ObjectId,
      medium: Schema.ObjectId,
      large: Schema.ObjectId,
      original: Schema.ObjectId
    },
    modified: { type: Date, 'default': Date.now }
});
    
// TODO: The error should probably be reported outwards 
function _genericSaveError(schema) {
  return function(err, object) {
    if (err) {
      console.error('Unable to save to ' + schema);
      console.error('Error: ' + err);
      return;
    }

    console.log("saved %s: id=%s", schema, object._id);
  }
}
    
function GalleryDB(opts) {
   
   var host = opts.host || 'localhost';
   var dbname = opts.dbname || 'photo_gallery_db';
   var port = opts.port || 27017;

   this.connection = mongoose.createConnection(host, dbname, port, opts);
   
   this.User = this.connection.model('User', UserSchema);
   this.Photo = this.connection.model('Photo', PhotoSchema);
   this.PhotoImage = this.connection.model('PhotoImage', PhotoImageSchema);
};

GalleryDB.prototype.insertUser = function(userTransport) {
  var user = new this.User(userTransport);
  
  user.save(_genericSaveError('User'));
  
  return user;
}; 

GalleryDB.prototype.findUserByUsername = function(username, callback) {
  this.User.findOne({ username: username }, function (err, doc){
    // doc is a Document
    if (err) {
      console.log(err.errors);
    }
    
    callback(err, doc);
  });
};

GalleryDB.prototype.insertPhoto = function(photoTransport) {
  // cascase save
  for (imageName in photoTransport.images) {
     var photoImage =  new this.PhotoImage(photoTransport.images[imageName]);
     photoImage.save(_genericSaveError('PhotoImage'));  
     photoTransport.images[imageName] = photoImage;
  }
  var photo = new this.Photo(photoTransport);

  photo.save(_genericSaveError('Photo'));

  return photo;
};

GalleryDB.prototype.findPhotoImageById = function(id, callback) {
  this.PhotoImage.findOne({_id: id}, callback);
}


/**
 * Warning this is a pretty severe method!
 */
GalleryDB.prototype.dropAll = function(callback) {
   for ( collection in this.connection.collections ) { 
      this.connection.collections[collection].drop();
   } 
   callback();
};

GalleryDB.prototype.disconnect = function(callback) {
  this.connection.close(callback);
};

module.exports = {
  GalleryDB: GalleryDB
};