var vows = require('vows'),
    assert = require('assert'),
    gallerylib = require('../lib/gallery');

    // performs 6 operations     
function _compareUsers(expected, actual) {
  assert.equal(expected.username, actual.username); 
  assert.equal(expected.email, actual.email);
  assert.equal(expected.displayName, actual.displayName);
  assert.equal(expected.description, actual.description);
  assert.equal(expected.location, actual.location);
  assert.equal(expected.thumbnail, actual.thumbnail);
}  

vows.describe("GalleryManager").addBatch({
    "A GallaryManager" : {
        topic: function() {
            var gm = new gallerylib.GalleryManager('restgallery-test-db');
            gm.createUser('photouser', 'photouser@example.com');
            return gm;
        },

        teardown: function (galleryManager) {
            galleryManager.dropData(function() {
                galleryManager.shutdown();
            });
        },

        "should manage users" : {
            topic: function (galleryManager) {
                return galleryManager.createUser('joeuser', 'joeuser@example.com');
            },
            "it should be inserted " : function(user)  {
                assert.equal(user.username, 'joeuser'); 
                assert.equal(user.email, 'joeuser@example.com');
            },
            "calling findUserByUsername" : {
                topic: function(user, galleryManager) {
                    var self = this;
                    galleryManager.findUserByUsername('joeuser', function(err, found) {self.callback(user, found);});
                },
                'should find the user': function(originalUser, foundUser) {
                    assert.isNotNull(foundUser);
                    _compareUsers(originalUser, foundUser);
                }
            }
        },

        "with a given user": {
            topic: function (galleryManager) {
                var self = this;
                galleryManager.findUserByUsername('photouser', function (err, user) {
                    self.callback(err, user, galleryManager);
                });
            },

            "given an uploaded photo": {

                topic: function(user, galleryManager) {
                    var self = this;
                    galleryManager.upload(user, './testdata/2000px-SMPTE_Color_Bars.png', 
                        function(err, photo) {
                            self.callback(err, photo, user, galleryManager)
                        });
                },

                "it should insert the photo": function(err, photo) {
                    assert.isNotNull(photo);
                    assert.isNotNull(photo.images);
                    
                },

                "and configured the image correctly" :  {
                    topic: function(photo, user, galleryManager) {
                        galleryManager.findPhotoImageById(photo.images.original, this.callback);
                    },

                    "it should have the correct sizes" : function (err, photoImage) {
                        assert.isNotNull(photoImage);
                        assert.equal(photoImage.width, 2000);
                        assert.equal(photoImage.height, 1500);                      
                    }
                }
            }
        }
    }
      

}).export(module);